// A BCPL, ASCII MANDELBROT

// This implementation copyright (c) 2019, Gordon Henderson
// ... Updated 2023 because I could.

// Permission to use/abuse anywhere for any purpose granted, but
// it comes with no warranty whatsoever. Good luck!

// BCPL translation of my BASIC Mandelbrot on a line by line basis.

GET "libhdr.h"
GET "sys.h"

LET start () = VALOF
{
  LET cols = ".,'~=+:;[/<&?oxOX# "
  LET mi = cols%0
  LET s0 = 1

  LET mx =        4.0
  LET ls = 0.0 #- 2.0
  LET tp =        1.25
  LET xs =        2.5
  LET ys = 0.0 #- 2.5
  LET w  = 64
  LET h  = 48

  LET sx = xs #/ (FLOAT w)
  LET sy = ys #/ (FLOAT h)

  LET tStart, tEnd = ?, ?

  tStart := sys (Sys_cputime)

  FOR y = 0 TO h DO
  {
    LET cy = (FLOAT y) #* sy #+ tp
    LET x2, y2 = ?, ?

    FOR x = 0 TO w DO
    {
      LET cx = (FLOAT x) #* sx #+ ls
      LET cc = s0
      LET t = ?
      LET zx, zy = 0.0, 0.0

L370:
      x2 := zx #* zx
      y2 := zy #* zy

      IF cc > mi          THEN GOTO L460
      IF (x2 #+ y2) #> mx THEN GOTO L460

      t  := x2 #- y2 #+ cx
      zy := (zx #+ zx) #* zy #+ cy
      zx := t
      cc := cc + 1
      GOTO L370

L460:
      wrch (cols%(cc - s0))
    }
    newline ()
  }

  tEnd := sys (Sys_cputime)

  writef ("Time taken: %5.3d seconds*n", tEnd - tStart)

  RESULTIS 0
}
