// A BCPL, ASCII MANDELBROT

// This implementation copyright (c) 2019, Gordon Henderson
// ... Updated 2023 because I could.

// Permission to use/abuse anywhere for any purpose granted, but
// it comes with no warranty whatsoever. Good luck!

// This version using scaled integers but is otherwise a more or
//	less exact implementation of the BASIC one that uses floating
//	point numbers.

// Essentially we scale by 4.

GET "libhdr.h"
GET "sys.h"


LET start() = VALOF
{
  LET cx,cy = ?,?

  LET xmin,xmax,ymin,ymax,maxiter,dx,dy = ?,?,?,?,?,?,?

  LET tEnd, tStart = ?, ?
  LET cols = ".,'~=+:;[/<&?oxOX# "

  tStart := sys (Sys_cputime)

  xmin := -8192
  xmax :=  2048
  ymin := -5124
  ymax :=  5124

  maxiter := cols%0

  dx := (xmax - xmin) / 64
  dy := (ymax - ymin) / 48

  cy := ymin
  WHILE cy <= ymax DO
  {
    cx := xmin
    WHILE cx <= xmax DO
    {
      LET q=?
      LET x,y,x2,y2,iter = 0,0,0,0,0

      WHILE iter < maxiter DO
      {
        IF x2+y2 > 16384 THEN
          BREAK

        q := x*y
        TEST q < 0 THEN 
          y := (-(-q >> 11)) + cy
        ELSE
          y := (   q >> 11)  + cy

        x    := x2 - y2 + cx
        x2   := (x*x) >> 12
        y2   := (y*y) >> 12

        iter := iter + 1
      }
      wrch (cols%(iter))
      cx := cx + dx
    }
    wrch ('*n')

    cy := cy + dy
  }

  tEnd := sys (Sys_cputime)

  writef ("Time taken: %5.3d seconds*n", tEnd - tStart)

  RESULTIS 0
}
